#################################################################
# R script for investigating hierarchical aspects in friendship #
#################################################################
#

library(RSiena)
setwd("<put the appropriate address here>")

# Read some of the Glasgow data sets:

friend1 <- as.matrix(read.table("Glasgow-1-net.dat"))
friend2 <- as.matrix(read.table("Glasgow-2-net.dat"))
friend3 <- as.matrix(read.table("Glasgow-3-net.dat"))
demographics <- as.matrix(read.table("Glasgow-sex-birthyear.dat"))

# recode and identify missing values (see 'Glasgow-readme.txt'):

friend1[friend1 == 2] <- 1
friend2[friend2 == 2] <- 1
friend3[friend3 == 2] <- 1
friend1[friend1 %in% c(9,10)] <- NA
friend2[friend2 %in% c(9,10)] <- NA
friend3[friend3 %in% c(9,10)] <- NA
demographics[demographics==0] <- NA

# =========================================
# Check degree-based global hierarchy idea
# =========================================
#
# By way of example for network 1 only:

odegree <- rowSums(friend1,na.rm=TRUE)
idegree <- colSums(friend1,na.rm=TRUE)

# correlate them to see if there is vertical hierarchy:

cor(odegree,idegree)

# gives a positive number! Does that mean "no hierarchy"?
#
# Idea: control for 'reciprocal degree' (horizontal dtructure).
#
# First calculate symmetrised matrix, then
# recode missings as they should be:

Rfriend1 <- symmetrize(friend1,rule="strong")
Rfriend1[is.na(friend1)] <- NA
Rfriend1[is.na(t(friend1))] <- NA

# now calculate reciprocal degree and residual in- and outdegree:

rdegree <- rowSums(Rfriend1,na.rm=TRUE)
residegree <- idegree-rdegree
resodegree <- odegree-rdegree

# correlate these, to see if there is evidence for 'residual' vertical hierarchy:

cor(resodegree,residegree)

# now gives a negative number! Hierarchy seems to be there, 
# but is "hidden" in horizontal relations.
#
# Now let's move on to assignment on hierarchy.
# Once more read & recode data (because code 10 is not really missing, but absent!).

friend1 <- as.matrix(read.table("Glasgow-1-net.dat"))
friend2 <- as.matrix(read.table("Glasgow-2-net.dat"))
friend3 <- as.matrix(read.table("Glasgow-3-net.dat"))
demographics <- as.matrix(read.table("Glasgow-sex-birthyear.dat"))

# recode and identify missing values (see 'Glasgow-readme.txt'):

friend1[friend1 == 2] <- 1
friend2[friend2 == 2] <- 1
friend3[friend3 == 2] <- 1
friend1[friend1 == 9] <- NA
friend2[friend2 == 9] <- NA
friend3[friend3 == 9] <- NA

# extract sex variable from demographics matrix:
sex.F <- demographics[,1]

# Now we can get the boys-only network by indexing only boys, etc.

friend1[boys,boys][1:5,1:5] # first five boys' subnetwork of friend1
friend1[girls,girls][1:5,1:5] # first five girls' subnetwork of friend1

# Now proceed in this vein "as usual" for RSiena projects:

# Identify dependent network variable:
friendship.b <- sienaNet(array(c(friend1[boys,boys],friend2[boys,boys],friend3[boys,boys]),dim=c(sum(boys),sum(boys),3)))
friendship.g <- sienaNet(array(c(friend1[girls,girls],friend2[girls,girls],friend3[girls,girls]),dim=c(sum(girls),sum(girls),3)))

# Bind data together for Siena analysis:
data.b <- sienaDataCreate(friendship.b)
data.g <- sienaDataCreate(friendship.g)

# Create effects objects:
effects.b <- getEffects(data.b)
effects.g <- getEffects(data.g)

# Write first descriptive results to protocol file (optional):
print01Report(data.b,effects.b,modelname = 'boys-init')
print01Report(data.g,effects.g,modelname = 'girls-init')

# Specify the model according to the assignment:
effectsDocumentation() # to get shortnames
effects.b <- includeEffects(effects.b,transTrip,cycle3,transTies,outPop)
effects.g <- includeEffects(effects.g,transTrip,cycle3,transTies,outPop)
effects.b

# Now create model objects:
model.b <- sienaModelCreate(useStdInits = TRUE, projname = 'boys-results')
model.g <- sienaModelCreate(useStdInits = TRUE, projname = 'girls-results')

# Estimate the model:
results.b <- siena07(model.b, data=data.b, effects=effects.b, batch=FALSE, verbose=FALSE)
results.g <- siena07(model.g, data=data.g, effects=effects.g, batch=FALSE, verbose=FALSE)
